/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.crop.type;

import java.io.IOException;
import java.util.Date;
import nl.cloudfarming.client.crop.model.CropService;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropProvider;
import nl.cloudfarming.client.crop.model.event.CropProviderEventKey;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.netbeans.api.project.Project;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.modules.ModuleInfo;
import org.openide.modules.ModuleInstall;
import org.openide.modules.Modules;
import org.openide.util.*;
import org.openide.util.lookup.ServiceProvider;

/**
 * Crop loader class Will start on application startup Will look for all
 * implementations of CropProvider registered as service provider of
 * CropProvider Is registered as LookupListener on CropProvider
 *
 * @author Merijn Zengers
 */
@ServiceProvider(service = CropsLoader.class)
public class CropsLoader implements LookupListener {
    //12 Hours
    protected static final long UPDATE_INTERVAL = 43200000;
    private static final String MODULE_NAME;
    private static final ModuleInfo info;
    private CropService cropService;
    protected static final String LAST_UPDATED_PROPERTY_NAME = "cropsLastUpdated"; 
    private static final long DEFAULT_LAST_UPDATED_VALUE = 0;
    
    static {
        info = Modules.getDefault().ownerOf(CropsLoader.class);        
        MODULE_NAME = info.getDisplayName();
    }
    
    public CropsLoader() {        
        loadCrops();
        GuiEventBus.addListener(new GuiEventListener() {
            
            @Override
            public void onEvent(GuiEvent event) {
                loadCrops((String) event.getContent());
            }
            
            @Override
            public boolean listensTo(GuiEventKey guiEventKey) {
                return guiEventKey.equals(CropProviderEventKey.UPDATED);
            }
            
            @Override
            public String getModuleName() {
                return MODULE_NAME;
            }
        });
        
        Lookup.getDefault().lookupResult(CropProvider.class).addLookupListener(this);
    }

    /**
     * Load crops from all the available CropProvider implementation
     */
    public final void loadCrops() {
        for (CropProvider provider : Lookup.getDefault().lookupAll(CropProvider.class)) {
            loadCrops(provider);
        }
    }
    
    public void loadCrops(Project project) {
        for (CropProvider provider : Lookup.getDefault().lookupAll(CropProvider.class)) {
            for (Crop crop : provider.getCrops()) {
                try {
                    getCropService().addCrop(project, crop);
                } catch (DataObjectNotFoundException ex) {
                    Exceptions.printStackTrace(ex);
                } catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }
        }
    }

    /**
     * Load crops from a certain CropProvider implementation specified by the id
     *
     * @param cropProviderId the id of the CropProvider to load the crops from
     */
    public void loadCrops(String cropProviderId) {
        for (CropProvider provider : Lookup.getDefault().lookupAll(CropProvider.class)) {
            if (provider.getId().equals(cropProviderId)) {
                loadCrops(provider);
            }
        }
    }

    /**
     * Load crops from the specified CropProvider implementation
     *
     * @param cropProvider the CropProvider to load the crops from
     */
    public void loadCrops(CropProvider cropProvider) {
        if (cropProvider.getLastUpdated() > getStoredLastUpdated() + UPDATE_INTERVAL) {
            
            for (Crop crop : cropProvider.getCrops()) {
                try {
                    getCropService().addCrop(crop);
                } catch (DataObjectNotFoundException ex) {
                    Exceptions.printStackTrace(ex);
                } catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }
        }
    }
    
    protected CropService getCropService() {
        if (cropService == null) {
            cropService = Lookup.getDefault().lookup(CropService.class);
        }
        return cropService;
    }
    
    /**
     * Returns when the  services where last updated
     * @return long the time since last update in milliseconds from epoch
     */
    long getStoredLastUpdated(){
        return NbPreferences.forModule(this.getClass()).getLong(LAST_UPDATED_PROPERTY_NAME, DEFAULT_LAST_UPDATED_VALUE);
    }
    
    @Override
    public void resultChanged(LookupEvent le) {
        loadCrops(Lookup.getDefault().lookup(CropProvider.class));
    }
}
